@@ -48,8 +48,128 @@ class ConvDropoutNormNonlin(nn.Module):
         return self.lrelu(self.instnorm(x))
 
 
+class Deep_Supervision_UNet3d_V2(nn.Module):
+    def __init__(self, n_modalities = 4, final_output_channels = 5, patch_size = (128, 128, 128), is_binary = False):
+        super().__init__()
+        self.patch_size = np.array(patch_size)
+        self.networks_depth = self.get_networks_depth()
+        self.encoder_blocks = []
+        self.decoder_blocks = []
+        self.upsample_operations = []
+        self.final_output_channels = final_output_channels
+        self.is_binary = is_binary
+        
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
+        self.encoder1 = nn.Sequential(
+            ConvDropoutNormNonlin(n_modalities, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(32, 32, conv_kwargs = conv_kwargs1, conv_op = nn.Conv3d)
+        )
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 2}
+        conv_kwargs2 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
+        self.encoder2 = nn.Sequential(
+            ConvDropoutNormNonlin(32, 64, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(64, 64, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.encoder3 = nn.Sequential(
+            ConvDropoutNormNonlin(64, 128, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(128, 128, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.encoder4 = nn.Sequential(
+            ConvDropoutNormNonlin(128, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(256, 256, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.encoder5 = nn.Sequential(
+            ConvDropoutNormNonlin(256, 320, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(320, 320, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1,  'stride' : 1}
+        conv_kwargs2 = {'kernel_size' : 3, 'stride' : 1, 'padding':1}
+        transp_kwargs = {'kernel_size' : 2, 'stride' : 1, 'stride' : 2}
+
+
+        self.decoder1_trans = ConvDropoutNormNonlin(320, 256, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder1_conv = nn.Sequential(
+            ConvDropoutNormNonlin(256*2, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(256, 256, conv_kwargs = conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.decoder2_trans = ConvDropoutNormNonlin(256, 128, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder2_conv = nn.Sequential(
+            ConvDropoutNormNonlin(128*2, 128 , conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(128, 128, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.decoder3_trans = ConvDropoutNormNonlin(128, 64, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder3_conv = nn.Sequential(
+            ConvDropoutNormNonlin(64*2, 64, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(64, 64, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.decoder4_trans = ConvDropoutNormNonlin(64, 32, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder4_conv = nn.Sequential(
+            ConvDropoutNormNonlin(32*2, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(32, 32, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+
+        self.final_layer1 = nn.Sequential(
+            nn.Conv3d(32, final_output_channels, 3, 1, 1),
+            nn.InstanceNorm3d(final_output_channels),
+            nn.LeakyReLU(),
+            )
+        self.final_layer2 = nn.Sequential(
+            nn.Conv3d(64, final_output_channels, 3, 1, 1),
+            nn.InstanceNorm3d(final_output_channels),
+            nn.LeakyReLU(),
+            )
+        self.final_layer3 = nn.Sequential(
+            nn.Conv3d(128, final_output_channels, 3, 1, 1),
+            nn.InstanceNorm3d(final_output_channels),
+            nn.LeakyReLU(),
+            )
+        self.final_layer4 = nn.Sequential(
+            nn.Conv3d(256, final_output_channels, 3, 1, 1),
+            nn.InstanceNorm3d(final_output_channels),
+            nn.LeakyReLU(),
+            )
+
+        self.upsample4 = nn.Upsample(scale_factor=8, mode = 'trilinear')
+        self.upsample3 = nn.Upsample(scale_factor=4, mode = 'trilinear')
+        self.upsample2 = nn.Upsample(scale_factor=2, mode = 'trilinear')
+        
+    
+    def forward(self, x):
+        bs, nc, nx, ny, nz  = x.shape
+        x1 = self.encoder1(x)
+        x2 = self.encoder2(x1)
+        x3 = self.encoder3(x2)
+        x4 = self.encoder4(x3)
+        x5 = self.encoder5(x4)
+
+        x4_ = self.decoder1_conv(torch.cat((x4, self.decoder1_trans(x5)), dim = 1))
+        x3_ = self.decoder2_conv(torch.cat((x3, self.decoder2_trans(x4_)), dim = 1))
+        x2_ = self.decoder3_conv(torch.cat((x2, self.decoder3_trans(x3_)), dim = 1))
+        x1_ = self.decoder4_conv(torch.cat((x1, self.decoder4_trans(x2_)), dim = 1))
+
+        if not self.is_binary:
+            x1_ = F.softmax(self.final_layer1(x1_).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
+            x2_ = F.softmax(self.upsample2(self.final_layer2(x2_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
+            x3_ = F.softmax(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
+            x4_ = F.softmax(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
+        else:
+            x1_ = F.sigmoid(self.final_layer1(x1_).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x2_ = F.sigmoid(self.upsample2(self.final_layer2(x2_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x3_ = F.sigmoid(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x4_ = F.sigmoid(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+        
+        result = torch.stack([x1_, x2_, x3_, x4_]).permute(1,0,2,3,4,5)
+        
+        return result.contiguous()
+                
+    def get_networks_depth(self):
+        min_patch_axis = np.min(self.patch_size)
+        return np.clip(np.log2(min_patch_axis), 0, 6).astype(int)
+    
+
 class Deep_Supervision_UNet3d(nn.Module):
-    def __init__(self, n_modalities = 4, patch_size = (128, 128, 128), final_output_channels = 5, is_binary = False):
+    def __init__(self, n_modalities = 4, final_output_channels = 5, patch_size = (128, 128, 128), is_binary = False):
         super().__init__()
         self.patch_size = np.array(patch_size)
         self.networks_depth = self.get_networks_depth()
@@ -223,6 +343,66 @@ class Interactive_Model(nn.Module):
 
         return torch.sigmoid(x)
 
+class Daegu_Model10(nn.Module):
+    def __init__(self, patch_size):
+        super().__init__()
+        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
+        self.model = resnet10(**kwargs)
+        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
+
+    def forward(self, X):
+        X = self.model(X)
+        X = self.final_upsample(X)
+        return torch.softmax(X, dim = -1)
+
+class Daegu_Model34(nn.Module):
+    def __init__(self, patch_size):
+        super().__init__()
+        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
+        self.model = resnet34(**kwargs)
+        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
+
+    def forward(self, X):
+        X = self.model(X)
+        X = self.final_upsample(X)
+        return torch.softmax(X, dim = -1)
+
+class Daegu_Model101(nn.Module):
+    def __init__(self, patch_size):
+        super().__init__()
+        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
+        self.model = resnet101(**kwargs)
+        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
+
+    def forward(self, X):
+        X = self.model(X)
+        X = self.final_upsample(X)
+        return torch.softmax(X, dim = -1)
+
+class Daegu_Model200(nn.Module):
+    def __init__(self, patch_size):
+        super().__init__()
+        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
+        self.model = resnet200(**kwargs)
+        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
+
+    def forward(self, X):
+        X = self.model(X)
+        X = self.final_upsample(X)
+        return torch.softmax(X, dim = -1)
+
+class Daegu_Model50(nn.Module):
+    def __init__(self, patch_size):
+        super().__init__()
+        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
+        self.model = resnet50(**kwargs)
+        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
+
+    def forward(self, X):
+        X = self.model(X)
+        X = self.final_upsample(X)
+        return torch.softmax(X, dim = -1)
+
 
 
 def conv3x3x3(in_planes, out_planes, stride=1, dilation=1):
@@ -427,7 +607,7 @@ class ResNet(nn.Module):
 
 
 def resnet10(**kwargs):
-    """Constructs a ResNet-18 model.
+    """Constructs a ResNet-10 model.
     """
     model = ResNet(BasicBlock, [1, 1, 1, 1], **kwargs)
     return model
