@@ -58,11 +58,19 @@ class Cropping(object):
 class Resampling(object):
     def __init__(self, anisotropy_axis_index = None, target_spacing = [1.0, 1.0, 1.0], random = False, device = None):
         self.device = device
-        self.anisotropy_axis_index = anisotropy_axis_index
+        self.anisotropy_axis_index = self.new_anisotropy_axis(anisotropy_axis_index)
         target_spacing = self.new_spacing(target_spacing)
         self.target_spacing = np.array(target_spacing)
         self.random = random
     
+    def new_anisotropy_axis(self, array):
+        for i in range(len(array)):
+            if array[i] == 2:
+                array[i] = 0
+            else:
+                array[i] += 1
+        return array
+
     def new_spacing(self, spacing):
         new_spacing = np.zeros(3)
         new_spacing[0] = spacing[2]
@@ -81,7 +89,7 @@ class Resampling(object):
         original_spacing = self.new_spacing(original_spacing)
         if self.random:
             self.target_spacing = original_spacing * np.random.uniform(0.20, 5.0, size = np.array(original_spacing).shape)
-        new_shape = list((original_spacing[::-1]/self.target_spacing[::-1] * np.array([data.shape[-3], data.shape[-2], data.shape[-1]])).astype(int))
+        new_shape = list((original_spacing/self.target_spacing * np.array([data.shape[-3], data.shape[-2], data.shape[-1]])).astype(int))
         data = F.interpolate(data.unsqueeze(0), new_shape).squeeze(0)
 
         return data
